AtomicU16

Struct AtomicU16 

pub struct AtomicU16(/* private fields */);
Creusot wrapper around [std::sync::atomic::$atomic_type] Creusot wrapper around std::sync::atomic::AtomicU16.

Implementations

Source

impl AtomicU16

pub fn new(val: u16) -> (Self, Ghost<Perm<AtomicU16>>)

ensures

result.1.val() == val

ensures

*result.1.ward() == result.0

terminates

pub fn into_inner(self, own: Ghost<Perm<AtomicU16>>) -> u16

Wrapper for std::sync::atomic::AtomicU16::into_inner.

requires

self == *own.ward()

ensures

result == own.val()

pub fn compare_exchange<F>( &self, current: u16, new: u16, f: Ghost<F>, ) -> Result<u16, u16>
where F: FnGhost + FnOnce(Result<&mut Committer<Self, u16, SeqCst, SeqCst>, &Committer<Self, u16, SeqCst, None>>),

Wrapper for std::sync::atomic::AtomicU16::compare_exchange.

The load and the store are always sequentially consistent.

requires

forall<c: &mut Committer<Self, $type, ordering::SeqCst, ordering::SeqCst>>
    !c.shot_store() ==> c.ward() == *self ==>
    c.val_load().deep_model() == current.deep_model() ==>
    c.val_store() == new ==>
    f.precondition((Ok(c),)) && (f.postcondition_once((Ok(c),), ()) ==> (^c).shot_store())

requires

forall<c: &Committer<Self, $type, ordering::SeqCst, ordering::None>>
    !c.shot_store() ==> c.ward() == *self ==>
    // NOTE: This following line is not present for `weak`
    c.val_load().deep_model() != current.deep_model() ==>
    f.precondition((Err(c),))

ensures

match result {
    Ok(result) => {
        exists<c: &mut Committer<Self, $type, ordering::SeqCst, ordering::SeqCst>>
            !c.shot_store() && c.ward() == *self &&
            c.val_load().deep_model() == current.deep_model() &&
            c.val_store() == new &&
            result == c.val_load() &&
            f.postcondition_once((Ok(c),), ())
    },
    Err(result) => {
       exists<c: &Committer<Self, $type, ordering::SeqCst, ordering::None>>
            !c.shot_store() && c.ward() == *self &&
            // NOTE: This following line is not present for `weak`
            c.val_load().deep_model() != current.deep_model() &&
            result == c.val_load() &&
            f.postcondition_once((Err(c),), ())
    }
}
pub fn compare_exchange_weak<F>( &self, current: u16, new: u16, f: Ghost<F>, ) -> Result<u16, u16>
where F: FnGhost + FnOnce(Result<&mut Committer<Self, u16, SeqCst, SeqCst>, &Committer<Self, u16, SeqCst, None>>),

Wrapper for std::sync::atomic::AtomicU16::compare_exchange_weak.

The load and the store are always sequentially consistent.

requires

forall<c: &mut Committer<Self, $type, ordering::SeqCst, ordering::SeqCst>>
    !c.shot_store() ==> c.ward() == *self ==>
    c.val_load().deep_model() == current.deep_model() ==>
    c.val_store() == new ==>
    f.precondition((Ok(c),)) && (f.postcondition_once((Ok(c),), ()) ==> (^c).shot_store())

requires

forall<c: &Committer<Self, $type, ordering::SeqCst, ordering::None>>
    !c.shot_store() ==> c.ward() == *self ==>
    f.precondition((Err(c),))

ensures

match result {
    Ok(result) => {
        exists<c: &mut Committer<Self, $type, ordering::SeqCst, ordering::SeqCst>>
            !c.shot_store() && c.ward() == *self &&
            c.val_load().deep_model() == current.deep_model() &&
            c.val_store() == new &&
            result == c.val_load() &&
            f.postcondition_once((Ok(c),), ())
    },
    Err(result) => {
       exists<c: &Committer<Self, $type, ordering::SeqCst, ordering::None>>
            !c.shot_store() && c.ward() == *self &&
            result == c.val_load() &&
            f.postcondition_once((Err(c),), ())
    }
}
pub fn load<F>(&self, f: Ghost<F>) -> u16
where F: FnGhost + FnOnce(&Committer<Self, u16, SeqCst, None>),

Wrapper for std::sync::atomic::AtomicU16::load.

The load is always sequentially consistent.

requires

forall<c: &Committer<Self, $type, ordering::SeqCst, ordering::None>>
    !c.shot_store() ==> c.ward() == *self ==> f.precondition((c,))

ensures

exists<c: &Committer<Self, $type, ordering::SeqCst, ordering::None>>
    !c.shot_store() && c.ward() == *self && c.val_load() == result && f.postcondition_once((c,), ())
pub fn store<F>(&self, val: u16, f: Ghost<F>)
where F: FnGhost + FnOnce(&mut Committer<Self, u16, None, SeqCst>),

Wrapper for std::sync::atomic::AtomicU16::store.

The store is always sequentially consistent.

requires

forall<c: &mut Committer<Self, $type, ordering::None, ordering::SeqCst>>
    !c.shot_store() ==> c.ward() == *self ==> c.val_store() == val ==>
    f.precondition((c,)) && (f.postcondition_once((c,), ()) ==> (^c).shot_store())

ensures

exists<c: &mut Committer<Self, $type, ordering::None, ordering::SeqCst>>
    !c.shot_store() && c.ward() == *self && c.val_store() == val &&
    f.postcondition_once((c,), ())
Source

impl AtomicU16

pub fn fetch_add<F>(&self, val: u16, f: Ghost<F>) -> u16
where F: FnGhost + FnOnce(&mut Committer<Self, u16, SeqCst, SeqCst>),

Wrapper for std::sync::atomic::AtomicU16::fetch_add.

The load and the store are always sequentially consistent.

requires

forall<c: &mut Committer<Self, $int_type, ordering::SeqCst, ordering::SeqCst>>
    !c.shot_store() ==> c.ward() == *self ==> c.val_store() == val + c.val_load() ==>
    f.precondition((c,)) && (f.postcondition_once((c,), ()) ==> (^c).shot_store())

ensures

exists<c: &mut Committer<Self, $int_type, ordering::SeqCst, ordering::SeqCst>>
    !c.shot_store() && c.ward() == *self && c.val_store() == val + c.val_load() &&
    c.val_load() == result && f.postcondition_once((c,), ())